const httpProxy = require('http-proxy');
const http = require('http');
const debug = require('debug');
const MockProxyRulesSheetList = require('..');
const aRules = require('./data/a.rules.json');
const bRules = require('./data/b.rules.json');

const info = debug('server');
const error = debug('server:error');

module.exports = function createServer(port, cb) {
  // Create reverse proxy instance
  const proxy = httpProxy.createProxy({ secure: false, changeOrigin: true });
  const proxyRules = new MockProxyRulesSheetList();
  proxyRules.set('a', aRules);
  proxyRules.set('b', bRules);

  proxy.on('error', (e) => {
    error(e);
  });

  // Create http server that leverages reverse proxy instance
  // and proxy rules to proxy requests to different targets
  http.createServer((req, res) => {
    // a match method is exposed on the proxy rules instance
    // to test a request to see if it matches against one of the specified rules
    info(`${req.method} ${req.url}`);
    try {
      const target = proxyRules.match(req);
      if (!target) {
        throw new Error('The request url and path did not match any of the listed rules!');
      }
      info(`==> ${target}`);
      proxy.web(req, res, { target });
    } catch (err) {
      error(`==> ${err.message}`);
      res.writeHead(500, { 'Content-Type': 'text/plain' });
      res.end(err.message);
    }
  }).listen(port, cb);
};
